/*****************************
 * DreamShell ##version##    *
 * app.h                     *
 * DreamShell App            *
 * Created by SWAT           *
 * http://www.dc-swat.net.ru *
 ****************************/


#ifndef _DS_APP_H
#define _DS_APP_H


#include "ds.h"
#include <mxml/mxml.h>


#define APP_STATE_OPENED 0x00000001
#define APP_STATE_LOADED 0x00000002
#define APP_STATE_READY 0x00000003
#define APP_STATE_PROCESS 0x00000004
#define APP_STATE_SLEEP 0x00000005


typedef struct App {
    
    const char *fn;
    const char *name;
    const char *icon;
    const char *ver;
    int id;
    
    Item_list_t *resources;
    Item_list_t *elements;
    mxml_node_t *xml;
    
    GUI_Widget *body;
    
    Thread_t *thd;
    lua_State *lua;
    
    int state;

} App_t;



int InitApps();
void ShutdownApps();

Item_list_t *GetAppList();
App_t *GetAppById(int id);
App_t *GetAppByFileName(const char *fn);
App_t *GetAppByName(const char *name);
App_t *GetCurApp();

App_t *AddApp(const char *fn);
int RemoveApp(App_t *app);

int OpenApp(App_t *app);
int CloseApp(App_t *app, int unload);
int SwitchApp(App_t *old_app, App_t *new_app, int unload);

int SetAppSleep(App_t *app, int sleep);

int LoadApp(App_t *app, int build);
int UnLoadApp(App_t *app);


int BuildAppBody(App_t *app);
int AddToAppBody(App_t *app, GUI_Widget *widget);
int RemoveFromAppBody(App_t *app, GUI_Widget *widget);

int CallAppBodyEvent(App_t *app, char *event);
void WaitApp(App_t *app);

// Utils

typedef struct AppSwitch {

	App_t *old_app;
	App_t *new_app;
	int unload;

} AppSwitch_t;

typedef struct lua_callback_data {

       lua_State *state;
       char *script; 

} lua_callback_data_t;


typedef struct scrollbar_callback_data {

       GUI_Widget *scroll;
       GUI_Widget *panel; 

} scrollbar_callback_data_t;

char *FindXmlAttr(char *name, mxml_node_t *node, char *defValue);


#endif
